@@ -1,34 +1,30 @@
-/* 
-
-    TREELANG Compiler common definitions (treelang.h)
-
-    Copyright (C) 1986, 87, 89, 92-96, 1997, 1999, 2000, 2001, 2002, 2003, 2004,
-    2007  Free Software Foundation, Inc.
-
-    This program is free software; you can redistribute it and/or modify it
-    under the terms of the GNU General Public License as published by the
-    Free Software Foundation; either version 3, or (at your option) any
-    later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; see the file COPYING3.  If not see
-    <http://www.gnu.org/licenses/>.
-
-    In other words, you are welcome to use, share and improve this program.
-    You are forbidden to forbid anyone else to use, share and improve
-    what you give them.   Help stamp out software-hoarding!  
-
-    ---------------------------------------------------------------------------
-
-    Written by Tim Josling 1999, 2000, 2001, based in part on other
-    parts of the GCC compiler.
-
-*/
+/* TREELANG Compiler common definitions (treelang.h)
+   
+   Copyright (C) 1986, 87, 89, 92-96, 1997, 1999, 2000, 2001, 2002, 2003, 2004,
+   2007 Free Software Foundation, Inc.
+   
+   This program is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3, or (at your option) any
+   later version.
+   
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+   
+   You should have received a copy of the GNU General Public License
+   along with this program; see the file COPYING3.  If not see
+   <http://www.gnu.org/licenses/>.
+   
+   In other words, you are welcome to use, share and improve this program.
+   You are forbidden to forbid anyone else to use, share and improve
+   what you give them.   Help stamp out software-hoarding!  
+   
+   ---------------------------------------------------------------------------
+   
+   Written by Tim Josling 1999, 2000, 2001, based in part on other
+   parts of the GCC compiler.  */
 
 #include "input.h"
 
@@ -57,7 +53,7 @@ struct token_part GTY(())
   location_t location;
   unsigned int charno;
   unsigned int length; /* The value.  */
-  unsigned char* chars;
+  const unsigned char *chars;
 };
 
 /* Definitions for fields in production.  */
@@ -123,7 +119,7 @@ struct production_part GTY(())
 struct parameter_part GTY(())
 {
   struct prod_token_parm_item *next; /* Next entry.  */
-  unsigned char* variable_name; /* Name. */
+  const unsigned char *variable_name; /* Name. */
   tree * GTY ((skip)) where_to_put_var_tree; /* Where to save decl.  */
 };
 
